package massim.agent.framework;


import org.w3c.dom.Element;

import cz.agents.alite.communication.DefaultCommunicator;
import cz.agents.alite.communication.channel.CommunicationChannelException;
import cz.agents.alite.communication.channel.DefaultCommunicationChannelBroadcast;
import cz.agents.alite.communication.channel.DirectCommunicationChannelAsync;
import cz.agents.alite.communication.protocol.simple.SimpleProtocol;


public abstract class AbstractAsyncAgent extends AbstractAgent { // implements Runnable {

	private static String MASSIM_PROTOCOL_NAME = "MASSimProtocol";
	private static String BROADCAST_ADDRESS = DefaultCommunicationChannelBroadcast.BROADCAST_ADDRESS;
	
	private SimpleProtocol comm;
	private DefaultCommunicator communicator;
	
	public AbstractAsyncAgent(String username, String password) {

		super(username, password);
		
		communicator = new DefaultCommunicator(username);

		try {
			communicator.addChannel(new DirectCommunicationChannelAsync(communicator));
		} catch (CommunicationChannelException e) {
			e.printStackTrace();
		}
		
		comm = new SimpleProtocol(communicator, MASSIM_PROTOCOL_NAME) {
			
			@Override
			protected void processMessage(Object content) {
				onMessageReceivedPrivate((MessageContent) content);
			}
		};
	}

	// message arrival notification
	protected abstract void onMessageReceived(MessageContent content);

	// message arrival handler for the AsynchronousAgent class
	protected void onMessageReceivedPrivate(MessageContent content)
	{
		onMessageReceived(content);
	}

	
	private SimpleProtocol getComm() {
		return comm;
	}

	// public send message facility
	public void sendMessage(MessageContent content, String receiver) {
		getComm().sendMessage(content.clone(), receiver);
	}

	// public broadcast facility
	public void broadcastMessage(MessageContent content) {
		getComm().sendMessage(content.clone(), BROADCAST_ADDRESS);
	}
	
	protected final void processRequestAction(Element perception, Element target,
			long currenttime, long deadline) {
		// react to normal perception, modify target so that it becomes a valid action

		target.setAttribute("type", processRequestAction(new Perception(perception)));
	}

	public void processSimulationStart(Element perception, long currenttime) {
		Perception.setSimulationParams(perception);
	}

	public void processSimulationEnd(Element perception, long currenttime) {}
	
	protected abstract String processRequestAction(Perception perception);
}
