package massim.agent.implementation;

import massim.agent.framework.AbstractDamaSyncAgent;
import massim.agent.framework.MessageContent;

import org.w3c.dom.Element;

public class DamaSyncAgent extends AbstractDamaSyncAgent {

    /**
     * Simulation has finished
     */
    public Boolean bSimFinished = false;
    /**
     * We have reached target destination
     */
    public Boolean bTargetReached = false;
    /**
     * Initialization has been done
     */
    public Boolean bInitialized = false;
    /**
     * Target X coordination
     */
    public Integer targetX = -1;
    /**
     * Sleep time before inbox is read
     * (we wait for other messages to read them in the right order)
     */
    public Integer sleepTimeBeforeInboxRead = 50;
    /**
     * Sleep time for the agent's control loop
     */
    public Integer sleepToSlowDown = 500;

    /**
     * Dama agent constructor
     * 
     * @param username User name
     * @param password Password
     * @param priority Agent's priority (unique integer number)
     */
    public DamaSyncAgent(String username, String password, Integer priority) {
        super(username, password, priority);
    }

    /**
     * Main control loop of the agent
     */
    @Override
    protected void runSyncAgent() {

        Boolean bSimEnd = false;
        while (!bSimEnd) { // until the end of the simulation

            // MAIN CONTROL LOOP //

            // initialization
            init();

            // perception of location, read messages
            perceive();

            // deliberate and act (move/send messages)
            deliberateAndAct();

            // slow down the control loop
            sleep(sleepToSlowDown);

            // check if it is not the end of the simulation
            synchronized (this) {
                bSimEnd = this.bSimFinished;
            }
        }
    }

    /**
     * Handle perception - location and incoming messages
     */
    private void perceive() {

        // LOCATION //
        // Position of the agent is automatically stored in variables currentX and currentY
        // System.out.println("PERCEPTION of agent" + priority + ": step #" + getSimStep() + " currentX:" + currentX + " currentY:" + currentY);

        // READ NEW MESSAGES IN MAILBOX //
        if (!isEmptyInbox()) {
            // WAIT //
            // Wait for receiving other messages to be able to read them in the right order
            sleep(sleepTimeBeforeInboxRead);

            // READ //
            // Pop a message from inbox
            MessageContent msg = popInboxMessage();

            // TODO: Add your code here //






            // Debug info
            if (priority == 1 || priority == 3) {
                System.out.println("RECEIVED by agent" + priority + " step=#" + getSimStep() + " timestamp=" + msg.getTimestamp() + " MSG='" + msg.toString() + "'");
            }

            //////////////////////////////            
        }
    }

    /**
     * Deliberate and act
     */
    private void deliberateAndAct() {

        // TODO: Add your code here //

        // DELIBERATE //
        

         

        // SEND MESSAGES //
        
        // Example: send messages to other agents
        if (priority == 2) {
            sendHandleOK(1); // send handle OK to agent 1
            sendHandleNoGood(3, "nogood"); // send handle nogood to agent 3
            Thread.yield();
        }

        // MOVE //
        // Example: move towards targetX
        if (currentX < targetX) {
            actRight();
        } else {
            if (currentX > targetX) {
                actLeft();
            } else {
                actNothing();
            }
        }

        //////////////////////////////            
    }

    /**
     * Handle the initialization of the agent
     */
    private void init() {
        if (!bInitialized) {

            // TODO: Add your code here //
            targetX = 3;





            //////////////////////////////

            // initialization is done
            bInitialized = true;
        }
    }

    /**
     * Called as soon as agent logged in successfully
     */
    @Override
    public void processLogIn() {
        super.processLogIn();
    }

    /**
     * React on simulation start
     */
    @Override
    public void processSimulationStart(Element perception, long currenttime) {
        super.processSimulationStart(perception, currenttime);
    }

    /**
     * React on simulation end
     */
    @Override
    public void processSimulationEnd(Element perception, long currenttime) {
        super.processSimulationEnd(perception, currenttime);
        bSimFinished = true;
    }
}
